package cn.iocoder.yudao.module.delivery.controller.admin;

import cn.iocoder.yudao.framework.common.exception.ErrorCode;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.dict.core.dto.DictDataRespDTO;
import cn.iocoder.yudao.framework.excel.util.ExcelUtils;
import cn.iocoder.yudao.framework.mybatis.core.util.MyBatisUtils;
import cn.iocoder.yudao.framework.operatelog.core.annotations.OperateLog;
import cn.iocoder.yudao.framework.redis.helper.RedisHelper;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.delivery.convert.CustomerAnalysis;
import cn.iocoder.yudao.module.delivery.entity.*;
import cn.iocoder.yudao.module.delivery.entity.orderdata.view_order_businesstime;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService;
import cn.iocoder.yudao.module.delivery.service.CustomerAnalysisService2;
import cn.iocoder.yudao.module.delivery.service.EcwReportPermissionService;
import cn.iocoder.yudao.module.delivery.service.view_order_businesstimeService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.YearMonth;
import java.util.*;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.error;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;
import static cn.iocoder.yudao.framework.operatelog.core.enums.OperateTypeEnum.EXPORT;

/**
 * (CustomerAnalysis)客户分析报表
 *
 * @author lanbm
 * @since 2024-03-27 21:25:55
 */
@RestController
@RequestMapping("/Report/CustomerAnalysis")
public class CustomerAnalysisControl {

    @Autowired
    private CustomerAnalysisService customerAnalysisService;

    @Resource
    private CustomerAnalysisService2 customerAnalysisService2;

    //报表权限信息
    @Resource
    private EcwReportPermissionService ecwReportPermissionService;

    @Resource
    private RedisHelper redisHelper;

    @Resource
    private view_order_businesstimeService vieworderbusinesstimeService;

    private List<view_order_businesstime> listBData = null;
    private List<CustomerAnalysisResp> objListResult = null;


    /*
     数据字典项
     */
    @Autowired
    private DictDataApi dictDataApi;

    /*
     计算百分比
     lanbm 2024-04-10 add
     */
    private String getPercentage(BigDecimal part, BigDecimal total) {
        BigDecimal percentage = part.divide(total, 5,
                BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal("100"));
        // 输出百分比
        String sR = format(percentage) + "%";
        return sR;
    }

    /*
     格式化BigDecimal为字符串，保留两位小数
     lanbm 2024-04-10 add
     */
    private String format(BigDecimal value) {
        NumberFormat numberFormat = NumberFormat.getInstance();
        numberFormat.setMinimumFractionDigits(2);
        numberFormat.setMaximumFractionDigits(2);
        return numberFormat.format(value.doubleValue());
    }

    private String getCurDate() {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String currentDate = sdf.format(date);
        return currentDate;
    }

    private int getCurYear() {
        Calendar calendar = Calendar.getInstance();
        int year = calendar.get(Calendar.YEAR);
        return year;
    }

    /*
     获取数据查询条件
     */
    private CustomerAnalysisReq getDataWhere(CustomerAnalysisReq query) {
        if (query.getSearchDataType1() != null &&
                query.getSearchDataType2() != null &&
                query.getSearchDataType3() != null) {

        }
        return query;
    }

    /*
     对查询逻辑做处理 lanbm 2024-04-16 add
     */
    private CustomerAnalysisReq getReq(CustomerAnalysisReq Req) {
        //计算月份差
        long lc = monthDiff(Req.getSdate(), Req.getEdate());
        if (lc == 0) {
            lc = 1;
        }
        Req.setNMonth(Integer.parseInt(String.valueOf(lc)));

        int y = getCurYear();
        if (Req.getSdate() == null) {
            Req.setCurYear(String.valueOf(y));
            Req.setSdate(String.valueOf(y) + "-01-01");

            if (Req.getDuibiYear() == null) {
                Req.setDuibiYear(String.valueOf(y - 1));
            } else {
                Req.setSduibidate(Req.getDuibiYear() + "-01-01");
            }

        } else {
            //2024-03-01
            String sT = Req.getSdate();
            String s = sT.substring(4, 10);

            if (Req.getDuibiYear() == null) {
                int nTemp = Integer.parseInt(sT.substring(0, 4)) - 1;
                Req.setSduibidate(String.valueOf(nTemp) + s);
            } else {
                Req.setSduibidate(Req.getDuibiYear() + s);
            }
            Req.setCurYear(sT.substring(0, 4));
        }

        if (Req.getEdate() == null) {
            Req.setEdate(getCurDate());

            String s = Req.getEdate().substring(5, 7);
            int ld = getLastDay(Integer.parseInt(Req.getDuibiYear()), Integer.parseInt(s));
            Req.setEduibidate(Req.getDuibiYear() + "-" + s + "-" + String.valueOf(ld));

        } else {
            String sE = Req.getEdate();
            String sY = sE.substring(0, 4);

            //int ld = getLastDay(Integer.parseInt(sY), Integer.parseInt(s));
            //2024-03-01
            String s1 = sE.substring(4, 10);
            if (Req.getDuibiYear() == null) {
                int nTemp = Integer.parseInt(sY) - 1;
                Req.setEduibidate(String.valueOf(nTemp) + s1);
            } else {
                Req.setEduibidate(Req.getDuibiYear() + s1);
            }
        }
        return Req;
    }


    /*
    获取某年某月的最后一天 lanbm 2024-04-16 add
    */
    private int getLastDay(int year, int month) {
        YearMonth yearMonth = YearMonth.of(year, month);
        int lastDay = yearMonth.lengthOfMonth();
        return lastDay;
    }


    /*
     客户分析列表lanbm 2024-04-02 add
     xml动态查询参考网址
     https://blog.csdn.net/CYK_byte/article/details/128611104
     */
    @PostMapping("/getListPage")
    public CommonResult<PageResult<CustomerAnalysisResp>>
    getListPage(@RequestBody Map map) throws JsonProcessingException {
        //ecw_customer 的部门字段都为空 department 都为空，所以要关联客户经理所在的部门来查询
        //ecw_order_item  order_item_type 订单类型：1 普货 2 重货 3 泡货
        //ecw_order   order_type    订单类型：1 普货 2 重货 3 泡货
        // is_cargo_control 是否控货 1 控货，0不控货
        //同比增长速度=(本期发展水平-同期发展水平)/同期发展水平×100%
        //客户角色数据字典
        //客户表和数据字典表中的编码方式不一样，做关联查询有异常，为了不改变表的编码方式，在后台做转换

        //获取当前用户ID
        //先判断用户有什么报表权限
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId, loginUserId);
        List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
        if (objList == null || objList.size() == 0) {
            ErrorCode eCode = new ErrorCode(2024041504,
                    "当前用户未分配报表权限，请联系系统管理员处理。");
            return error(eCode);
        }

        System.out.println(map);
        CustomerAnalysisReq query = JSON.parseObject(JSON.toJSONString(map),
                CustomerAnalysisReq.class);

        query = getReq(query);
        query = getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        PageResult<CustomerAnalysisResp> pageResult = GetDataResult(query);

        return success(pageResult);
    }

    private PageResult<CustomerAnalysisResp> GetDataResult(CustomerAnalysisReq query)
            throws JsonProcessingException {

        //把model转 JSON字符串 lanbm
        //String jsonStr = JsonUtils.toJsonString(query);
        //JsonUtils.SaveLog(jsonStr);

        System.out.println("客户分析查询条件为:" + query);

        List<DictDataRespDTO> dicCustomer_type =
                dictDataApi.getDictDatas("customer_type");
        Map<String, String> mapDic = new HashMap<>();
        for (DictDataRespDTO d : dicCustomer_type
        ) {
            mapDic.put(d.getValue(), d.getLabel());
        }

        PageResult<CustomerAnalysisResp> pageResult =
                customerAnalysisService.getListPage(query);
        String sTemp = "";
        int nPm = query.getStart() + 1;
        for (CustomerAnalysisResp r : pageResult.getList()) {
            r.setPx(nPm);
            nPm++;

            //客户类型转换
            String sType = r.getType();
            if (sType == null) {
                r.setCustomerrole("");
            } else {
                r.setCustomerrole(mapDic.get(sType));
            }

            if (r.getAllsumvolume() == null ||
                    r.getAllsumvolume().compareTo(BigDecimal.ZERO) == 0) {

                r.setAllsumvolume(new BigDecimal(0));

                //计算月均
                r = calMonthAvg(query, r, true);

                //计算总值
                r = calAll(r, true);

                //计算海运数据
                r = cal1(query, r, true);
                //计算空运数据
                r = cal3(query, r, true);
                //计算重货数据
                r = calZH(r, true);
                //计算泡货数据
                r = calPH(r, true);
                //计算控货数据
                r = calKH(r, true);

            } else {
                //计算月均
                r = calMonthAvg(query, r, false);
                //计算总值
                r = calAll(r, false);
                //计算海运数据
                r = cal1(query, r, false);
                //计算空运数据
                r = cal3(query, r, false);
                //重货
                r = calZH(r, false);
                //计算泡货占比
                r = calPH(r, false);
                //计算控货占比
                r = calKH(r, false);
            }
            r = calPick(r, false);
        }
        return pageResult;
    }


    private CustomerAnalysisResp calMonthAvg(CustomerAnalysisReq query,
                                             CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
            r.setMonthAvg("0");
        } else {
            r.setMonthAvg(r.getAllsumvolume().divide(
                    new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP).toString());

        }

        //计算同比
        if (r.getAllsumvolumeTb() == null ||
                r.getAllsumvolumeTb().compareTo(BigDecimal.ZERO) == 0) {
            r.setMonthAvgTbShow("同期值为0");
        } else {
            sTemp = getPercentage(
                    (r.getAllsumvolume().subtract(r.getAllsumvolumeTb())),
                    r.getAllsumvolumeTb());
            r.setMonthAvgTbShow(sTemp);
        }
        return r;
    }

    /*
    总值计算
     */
    private CustomerAnalysisResp calAll(CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        if (r.getAllsumvolumeTb() == null ||
                r.getAllsumvolumeTb().compareTo(BigDecimal.ZERO) == 0) {
            r.setAllsumvolumeTbShow("同期V值为0");
        } else {
            //(本期-同期)/同期
            sTemp = getPercentage(
                    (r.getAllsumvolume().subtract(r.getAllsumvolumeTb())),
                    r.getAllsumvolumeTb());
            r.setAllsumvolumeTbShow(sTemp);
        }
        return r;
    }


    /*
     计算提货数据
     */
    private CustomerAnalysisResp calPick(CustomerAnalysisResp r, boolean bZzNull) {
        if (r.getChargequantity() == 0) {
            r.setThL("总箱数为0");
        } else {
            r.setThL(getIntPercentage(r.getPickquantity(), r.getChargequantity()));
        }
        return r;
    }

    /*
      处理海运数据
     */
    private CustomerAnalysisResp cal1(CustomerAnalysisReq query,
                                      CustomerAnalysisResp r, boolean bZzNull) {
        String sTemp = "";

        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        if (r.getSumvolume1() == null ||
                r.getSumvolume1().compareTo(BigDecimal.ZERO) == 0) {
            //海运V值
            r.setSumvolumeV1(new BigDecimal(0));
            //海运体积数
            r.setSumvolume1(new BigDecimal(0));
            if (bZzNull == true) {
                r.setSeaZb("总值为0");
                //海运月均
                r.setSeaMonthAvg("总值为0");
            } else {
                //海运占比
                r.setSeaZb("0");
                //海运月均
                r.setSeaMonthAvg("0");
            }

        } else {
            //设置海运V值
            r.setSumvolumeV1(r.getSumvolume1());

            if (bZzNull == true) {
                r.setSeaZb("总值为0");
            } else {
                r.setSeaZb(getPercentage(r.getSumvolumeV1(), r.getAllsumvolume()));
            }
            //月均V值 是体积
            r.setSeaMonthAvg(r.getSumvolume1().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP).toString());
        }

        if (r.getSumvolumeTb1() == null ||
                r.getSumvolumeTb1().compareTo(BigDecimal.ZERO) == 0) {
            r.setSumvolumeTbV1(new BigDecimal(0));
            r.setSumvolumeTbShow1("同期为0");
        } else {
            r.setSumvolumeTbV1(new BigDecimal(100).multiply(r.getSumvolumeTb1()));

            sTemp = getPercentage(
                    (r.getSumvolumeV1().subtract(r.getSumvolumeTbV1())),
                    r.getSumvolumeTbV1());
            r.setSumvolumeTbShow1(sTemp);
        }
        return r;
    }


    /*
     处理空运数据
     */
    private CustomerAnalysisResp cal3(CustomerAnalysisReq query,
                                      CustomerAnalysisResp r, boolean bZzNull) {

        String sTemp = "";
        if (bZzNull == true) {
            r.setAllsumvolume(new BigDecimal(0));
        }

        //当期值
        if (r.getSumweight3() == null ||
                r.getSumweight3().compareTo(BigDecimal.ZERO) == 0) {

            r.setSumweightV3(new BigDecimal(0));
            r.setSumweight3(new BigDecimal(0));

            if (bZzNull == true) {
                r.setAirZb("总值为0");
                r.setAirMothAvg("总值为0");
            } else {
                r.setAirZb("0");
                r.setAirMothAvg("0");
            }
        } else {
            r.setSumweightV3(r.getSumweight3().divide(new BigDecimal(100)));
            if (bZzNull == true) {
                r.setAirZb("总值为0");
            } else {
                r.setAirZb(getPercentage(r.getSumweightV3(), r.getAllsumvolume()));

            }
            //月均是 重量
            r.setAirMothAvg(r.getSumweight3().divide(new BigDecimal(query.getNMonth()), 2,
                    BigDecimal.ROUND_HALF_UP).toString());
        }

        //同期值
        if (r.getSumweightTb3() == null ||
                r.getSumweightTb3().compareTo(BigDecimal.ZERO) == 0) {
            r.setSumweightTb3(new BigDecimal(0));
            r.setSumweightTbV3(new BigDecimal(0));
            r.setSumweightTbShow3("同期值为0");
        } else {
            r.setSumweightTbV3(r.getSumweightTb3().divide(new BigDecimal(100)));

            sTemp = getPercentage(
                    (r.getSumweightV3().subtract(r.getSumweightTbV3())),
                    r.getSumweightTbV3());
            r.setSumweightTbShow3(sTemp);
        }
        return r;
    }

    /*
     计算重货数据
     */
    private CustomerAnalysisResp calZH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //总值为0的情况
            r.setAllsumvolume(new BigDecimal(0));
        }

        //重货 占比
        if (r.getWeightSumV() == null ||
                r.getWeightSumV().compareTo(BigDecimal.ZERO) == 0) {
            r.setWeightSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setWeithtSumZb("总值为0");
            } else {
                r.setWeithtSumZb("0");
            }

        } else {
            r.setWeithtSumZb(getPercentage(r.getWeightSumV(), r.getAllsumvolume()));
        }
        return r;
    }

    /*
     计算泡货数据
     */
    private CustomerAnalysisResp calPH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //泡货 占比
            r.setAllsumvolume(new BigDecimal(0));
        }
        if (r.getPhSumV() == null ||
                r.getPhSumV().compareTo(BigDecimal.ZERO) == 0) {

            r.setPhSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setPhSumZb("总值为0");
            } else {
                r.setPhSumZb("0");
            }

        } else {
            r.setPhSumZb(getPercentage(r.getPhSumV(), r.getAllsumvolume()));
        }

        return r;
    }

    /*
     计算控货数据
     */
    private CustomerAnalysisResp calKH(CustomerAnalysisResp r, boolean bZzNull) {
        if (bZzNull == true) {
            //控货 占比
            r.setAllsumvolume(new BigDecimal(0));
        }
        if (r.getKhSumV() == null ||
                r.getKhSumV().compareTo(BigDecimal.ZERO) == 0) {

            r.setKhSumV(new BigDecimal(0));
            if (bZzNull == true) {
                r.setKhSumZb("总值为0");
            } else {
                r.setKhSumZb("0");
            }
        } else {
            r.setKhSumZb(getPercentage(r.getKhSumV(),
                    r.getAllsumvolume()));
        }
        return r;
    }


    private String getIntPercentage(int number, int divisor) {
        // 计算百分比
        double percentage = number / (double) divisor;
        // 格式化输出百分比
        String formatted = String.format("%.2f%%", percentage);
        return formatted;
    }

    private int monthDiff(String dateString1, String dateString2) {
        // 将字符串转换为 LocalDate 对象
        LocalDate date1 = LocalDate.parse(dateString1);
        LocalDate date2 = LocalDate.parse(dateString2);

        // 获软时间1的年份和月份
        int year1 = date1.getYear();
        int month1 = date1.getMonthValue();
        // 获取时间2的年份和月份
        int year2 = date2.getYear();
        int month2 = date2.getMonthValue();

        // 计算时间1的总月数
        int totalMonths1 = year1 * 12 + month1;
        // 计算时间2的总月数
        int totalMonths2 = year2 * 12 + month2;
        // 计算总月数差
        int monthsDiff = totalMonths2 - totalMonths1;

        return monthsDiff;
    }


    private List<CustomerAnalysisResp> GetPageResult(
            List<CustomerAnalysisResp> list, int pageNum, int pageSize) {
        /*
        // 正序
        Collections.sort(list, (a, b) -> {
            return a.排序字段 - b.排序字段;
        });
        // 倒序
        Collections.sort(list, (a, b) -> {
            return b.排序字段 - a.排序字段;
        });*/
        // 总数
        int total = list.size();

        // 分页数据
        List<CustomerAnalysisResp> subList =
                list.stream().skip((pageNum - 1) * pageSize).limit(pageSize).collect(Collectors.toList());
        return subList;

    }

    @GetMapping("/exportExcel")
    @ApiOperation("导出客户分析数据")
    @OperateLog(type = EXPORT)
    public void export(HttpServletResponse response, @Valid CustomerAnalysisReq query) throws IOException {
        query = getReq(query);
        query = getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        PageResult<CustomerAnalysisResp> pageResult = GetDataResult(query);
        List<CustomerAnalysisResp> list = pageResult.getList();
        List<CustomerAnalysisExcelResp> listExel =
                CustomerAnalysis.INSTANCE.convertList(list);
        // 导出 Excel
        ExcelUtils.write(response, "客户分析报表.xls", "客户分析报表",
                CustomerAnalysisExcelResp.class, listExel);
    }


    @PostMapping("/getListPageWithRedis")
    public CommonResult<PageResult<CustomerAnalysisResp>>
    getListPageWithRedis(@RequestBody Map map) throws JsonProcessingException {
        //ecw_customer 的部门字段都为空 department 都为空，所以要关联客户经理所在的部门来查询
        //ecw_order_item  order_item_type 订单类型：1 普货 2 重货 3 泡货
        //ecw_order   order_type    订单类型：1 普货 2 重货 3 泡货
        // is_cargo_control 是否控货 1 控货，0不控货
        //同比增长速度=(本期发展水平-同期发展水平)/同期发展水平×100%
        //客户角色数据字典
        //客户表和数据字典表中的编码方式不一样，做关联查询有异常，为了不改变表的编码方式，在后台做转换

        //获取当前用户ID
        //先判断用户有什么报表权限
        Long loginUserId = SecurityFrameworkUtils.getLoginUserId();
        LambdaQueryWrapper<EcwReportPermission> lambdaQueryWrapper =
                new LambdaQueryWrapper();
        lambdaQueryWrapper.eq(EcwReportPermission::getUserId, loginUserId);
        List<EcwReportPermission> objList = ecwReportPermissionService.list(lambdaQueryWrapper);
        if (objList == null || objList.size() == 0) {
            ErrorCode eCode = new ErrorCode(2024041504,
                    "当前用户未分配报表权限，请联系系统管理员处理。");
            return error(eCode);
        }

        System.out.println(map);
        CustomerAnalysisReq query = JSON.parseObject(JSON.toJSONString(map),
                CustomerAnalysisReq.class);

        query = getReq(query);
        query = getDataWhere(query);
        System.out.println("处理后的查询条件：" + query);

        //把model转 JSON字符串 lanbm
        //String jsonStr = JsonUtils.toJsonString(query);
        //JsonUtils.SaveLog(jsonStr);

        System.out.println("客户分析查询条件为:" + query);

        List<DictDataRespDTO> dicCustomer_type =
                dictDataApi.getDictDatas("customer_type");
        Map<String, String> mapDic = new HashMap<>();
        for (DictDataRespDTO d : dicCustomer_type
        ) {
            mapDic.put(d.getValue(), d.getLabel());
        }

        ObjectMapper objectMapper = new ObjectMapper();
        String sList = redisHelper.get("view_order_businesstime");
        if (sList == null || sList.length() == 0) {
            listBData = vieworderbusinesstimeService.getOrderBusinessData();
            String json = objectMapper.writeValueAsString(listBData);
            redisHelper.set("view_order_businesstime", json);
        } else {
            if (listBData == null) {
                listBData = JSON.parseArray(sList, view_order_businesstime.class);
            }
        }

        String sList2 = redisHelper.get("CustomerAnalysisResp");
        if (sList2 == null || sList2.length() == 0) {
            objListResult = customerAnalysisService2.getList(query);
            String json = objectMapper.writeValueAsString(objListResult);
            redisHelper.set("CustomerAnalysisResp", json);
        } else {
            objListResult = JSON.parseArray(sList2,
                    CustomerAnalysisResp.class);

        }

        if (query.getCountry() != null && query.getCountry().length() > 0) {
            int cC = Integer.parseInt(query.getCountry());
            objListResult =
                    objListResult.stream().filter(c -> c.getCountryID() == cC).collect(Collectors.toList());
        }

        String sTemp = "";
        int nPm = 1;
        for (CustomerAnalysisResp r : objListResult) {
            r.setPx(nPm);
            nPm++;

            //客户类型转换
            String sType = r.getType();
            if (sType == null) {
                r.setCustomerrole("");
            } else {
                r.setCustomerrole(mapDic.get(sType));
            }

            String sID = r.getId();
            List<view_order_businesstime> lAll =
                    listBData.stream().filter(c -> c.getCustomerid() == sID).
                            collect(Collectors.toList());


            BigDecimal sumV =
                    lAll.stream().map(view_order_businesstime::getVz).
                            reduce(BigDecimal.ZERO, BigDecimal::add);

            r.setAllsumvolume(sumV);

            if (r.getAllsumvolume() == null ||
                    r.getAllsumvolume().compareTo(BigDecimal.ZERO) == 0) {

                r.setAllsumvolume(new BigDecimal(0));

                //计算月均
                r = calMonthAvg(query, r, true);

                //计算总值
                r = calAll(r, true);

                //计算海运数据
                r = cal1(query, r, true);
                //计算空运数据
                r = cal3(query, r, true);
                //计算重货数据
                r = calZH(r, true);
                //计算泡货数据
                r = calPH(r, true);
                //计算控货数据
                r = calKH(r, true);

            } else {
                //计算月均
                r = calMonthAvg(query, r, false);
                //计算总值
                r = calAll(r, false);
                //计算海运数据
                r = cal1(query, r, false);
                //计算空运数据
                r = cal3(query, r, false);
                //重货
                r = calZH(r, false);
                //计算泡货占比
                r = calPH(r, false);
                //计算控货占比
                r = calKH(r, false);

            }
            r = calPick(r, false);
        }

        IPage<CustomerAnalysisResp> mpPage = MyBatisUtils.buildPage(query);
        long total = objListResult.size();
        System.out.println("符合条件的客户总数：" + String.valueOf(total));

        List<CustomerAnalysisResp> list = GetPageResult(objListResult, query.getPageNo(), query.getPageSize());
        PageResult<CustomerAnalysisResp> pageResult = new PageResult<>(list, total, mpPage.getSize(),
                query.getPageNo(), (total + mpPage.getSize() - 1) / mpPage.getSize());

        return success(pageResult);


    }

}
